@@ -177,6 +177,12 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 		LefError("Unknown layer type \"%s\" for NEW route\n", token); 
 		continue;
 	    }
+            else if (routeLayer >= Num_layers)
+	    {
+		LefError("DEF file contains layer \"%s\" which is not allowed "
+			"by the layer limit setting of %d\n", token, Num_layers);
+		continue;
+	    }
 	    paintLayer = routeLayer;
 
 	    if (special == (char)1)
@@ -262,7 +268,8 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 		    LefError("Error: Via \"%s\" named but undefined.\n", token);
 		    paintLayer = routeLayer;
 		}
-		if ((special == (char)0) && (paintLayer >= 0)) {
+		if ((special == (char)0) && (paintLayer >= 0) &&
+				(paintLayer < Num_layers)) {
 
 		    newRoute = (SEG)malloc(sizeof(struct seg_));
 		    newRoute->segtype = ST_VIA;
@@ -286,8 +293,12 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 		    newRoute->next = routednet->segments;
 		    routednet->segments = newRoute;
 		}
-		else
-		    LefError("Via \"%s\" does not define a metal layer!\n", token);
+		else {
+		    if (paintLayer >= Num_layers)
+		        LefError("Via \"%s\" exceeds layer limit setting!\n", token);
+		    else
+		        LefError("Via \"%s\" does not define a metal layer!\n", token);
+		}
 	    }
 	    else
 		LefError("Via name \"%s\" unknown in route.\n", token);
@@ -403,7 +414,7 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 		      UserObs = drect;
 		   }
 		}
-		else if (paintLayer >= 0) {
+		else if ((paintLayer >= 0) && (paintLayer < Num_layers)) {
 		   newRoute = (SEG)malloc(sizeof(struct seg_));
 		   newRoute->segtype = ST_WIRE;
 		   newRoute->x1 = locarea.x1;
@@ -426,6 +437,9 @@ DefAddRoutes(FILE *f, float oscale, NET net, char special)
 		   newRoute->next = routednet->segments;
 		   routednet->segments = newRoute;
 		}
+		else if (paintLayer >= Num_layers) {
+		    LefError("Route layer exceeds layer limit setting!\n");
+		}
 	    }
 
 endCoord:
@@ -1730,6 +1744,7 @@ DefRead(char *inName)
     float oscale;
     double start, step;
     double llx, lly, urx, ury;
+    double dXlowerbound, dYlowerbound, dXupperbound, dYupperbound;
     char corient = '.';
     DSEG diearea;
 
@@ -1921,12 +1936,9 @@ DefRead(char *inName)
 			PitchX[curlayer + 1] = PitchX[curlayer];
 		    llx = start;
 		    urx = start + step * channels;
-		    // Fix, 5/24/2013:  Set bounds according to the tracks,
-		    // since we really don't care about the die area.  But,
-		    // we should make sure this is consistent across layers. . .
-		    // if (llx < Xlowerbound)
+		    if ((llx / oscale) < Xlowerbound)
 			Xlowerbound = llx / oscale;
-		    // if (urx > Xupperbound)
+		    if ((urx / oscale) > Xupperbound)
 			Xupperbound = urx / oscale;
 		}
 		else {
@@ -1937,9 +1949,9 @@ DefRead(char *inName)
 			PitchY[curlayer + 1] = PitchY[curlayer];
 		    lly = start;
 		    ury = start + step * channels;
-		    // if (lly < Ylowerbound)
+		    if ((lly / oscale) < Ylowerbound)
 			Ylowerbound = lly / oscale;
-		    // if (ury > Yupperbound)
+		    if ((ury / oscale) > Yupperbound)
 			Yupperbound = ury / oscale;
 		}
 		LefEndStatement(f);
@@ -1958,10 +1970,15 @@ DefRead(char *inName)
 		break;
 	    case DEF_DIEAREA:
 		diearea = LefReadRect(f, 0, oscale); // no current layer, use 0
-		Xlowerbound = diearea->x1;
-		Ylowerbound = diearea->y1;
-		Xupperbound = diearea->x2;
-		Yupperbound = diearea->y2;
+		dXlowerbound = diearea->x1;
+		dYlowerbound = diearea->y1;
+		dXupperbound = diearea->x2;
+		dYupperbound = diearea->y2;
+		/* Seed actual lower/upper bounds with the midpoint */
+		Xlowerbound = (diearea->x1 + diearea->x2) / 2;
+		Ylowerbound = (diearea->y1 + diearea->y2) / 2;
+		Xupperbound = Xlowerbound;
+		Yupperbound = Ylowerbound;
 		LefEndStatement(f);
 		break;
 	    case DEF_PROPERTYDEFINITIONS:
@@ -2043,6 +2060,16 @@ DefRead(char *inName)
 	Fprintf(stdout, "DEF read: Processed %d lines.\n", lefCurrentLine);
     LefError(NULL);	/* print statement of errors, if any, and reset */
 
+    /* If there were no TRACKS statements, then use the DIEAREA */
+    if (Xlowerbound == Xupperbound) {
+	Xlowerbound = dXlowerbound;
+	Xupperbound = dXupperbound;
+    }
+    if (Ylowerbound == Yupperbound) {
+	Ylowerbound = dYlowerbound;
+	Yupperbound = dYupperbound;
+    }
+
     /* Cleanup */
 
     if (f != NULL) fclose(f);
